/*
 * The UDS Collection
 * Copyright (C) 2000, 2001  Julian v. Bock
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *	(See file COPYING for more information)
 */

#ifndef _uds_socket_hh
#define _uds_socket_hh


// includes

#include <fstream>

using namespace std;


namespace uds
{

/** stream class for sockets.
	\author Julian v. Bock */
class SocketStream : public fstream
{
public:
	/** default constructor */
	SocketStream();
	
	/** \param fd socket file descriptor */
	explicit
	SocketStream( int fd );
	
	/** destructor */
	virtual
	~SocketStream();
	
	/** creates an unix domain socket
		\param n name */
	virtual void
	UnixDomainSocket( const char* n );
	
	/** creates an internet namespace socket
		\param p port */
	virtual void
	InetSocket( unsigned short p );
	
	/** enables the socket to accept connections
		\param n queue size for pending connection requests */
	virtual void
	Listen( unsigned n = 1 );
	
	/** accepts a conection to a server socket
		\return new socket (file) descriptor */
	virtual int
	Accept();
	
	/** connects to a local unix domain socket
		\param n socket name */
	virtual void
	Connect( const char* hn );
	
	/** connects to a [remote] host
		\param hn host (IP or name; as accepted by gethostbyname)
		\param p port */
	virtual void
	Connect( const char* hn, unsigned short p );
	
	/** closes a connection */
	virtual void
	Close();
	
	/** sets a new socket
		\param s new socket */
	virtual void
	Attach( int s );
	
	/** checks whether the socket is valid
		\return true if the socket is valid */
	bool
	Valid() const { return sock >= 0; }
	
	/** \return socket file descriptor */
	int
	operator () () { return sock; }
	
private:
	/// socket file descriptor
	int	sock;
};


/** Creates a named unix domain socket.
	\param name socket name
	\return socket file descriptor */
int
UnixDomainSocket( const char *name );

/** Creates a local internet namespace socket
	\param p port number */
int
InetSocket( unsigned short p );

}

#endif // _uds_socket_hh
